unit FrameProcessing;
{
  For frame 
}
interface
const
  algFrameWidth=320;
  algFrameHeight=240;
  LastCol=algFrameWidth-1;
  LastRow=algFrameHeight-1;
  TotalItemsInFrame=algFrameHeight*algFrameWidth;
  LastItemInLine=TotalItemsInFrame-1;
type
  TRGB = record
    r,g,b:byte;
  end;
  T352x288RGBArray = array [0..LastRow,0..LastCol] of TRGB;
  p352x288RGBArray = ^T352x288RGBArray;
  TLineRGBArray = array [0..LastItemInLine] of TRGB;
  pLineRGBArray = ^TLineRGBArray;
  T352x288SmallIntArray = array [0..LastRow,0..LastCol] of SmallInt; //   
  TLineSmallIntArray = array [0..LastItemInLine] of SmallInt;
  pLineSmallIntArray = ^TLineSmallIntArray;
  TSearchingImage = record
    Image:T352x288SmallIntArray;
    Width:Integer;
    Height:Integer;
  end;
var
  //   ,      TDIB.
  RGBBuff:T352x288RGBArray;
  //  
  SmallIntBuff:T352x288SmallIntArray;
  //      
  //       (     
  //   . 
  SearchingImage:TSearchingImage;
  //   
  IsProcessing:boolean;

//    /  
procedure LoadRGBArray(const Data:p352x288RGBArray);
procedure RGB2SmallIntArrayConvert; // ,   RGBBuff  
procedure SmallInt2RGBArrayConvert; // ,   SmallIntBuff  
procedure StoreRGBArray(const Data:p352x288RGBArray);

//    
procedure FillSearchingImage (X,Y,Width,Height:Integer);
function CalculateBrightnessError (X,Y:Integer):Integer; // return error
function FindMinimumBrightnessErrorAtXYR(X1,Y1,R:Integer;var X,Y:Integer):Integer; // return minimum error
function FindMinimumBrightnessErrorAtAll(var X,Y:Integer):Integer; // return minimum error

//   
procedure Differences01;
procedure Differences02;
procedure FindMinMax(var Minimum,Maximum:SmallInt);
procedure ShowNormDelta (ZeroLevel:SmallInt);
procedure MinThreshold (Threshold:SmallInt);

//   
procedure TranslateXY (var X,Y:Integer;Width,Height:Integer);
procedure SwapAB (var a,b:Integer);

implementation
uses Math;

var
  TmpBuff:T352x288SmallIntArray; // temporary buffer

procedure TranslateXY (var X,Y:Integer;Width,Height:Integer);
begin
  X:=round(X*(algFrameWidth/Width));
  Y:=round(Y*(algFrameHeight/Height));
end;

procedure SwapAB (var a,b:Integer);
var
  c:Integer;
begin
  c:=a;
  a:=b;
  b:=c;
end;

procedure FillSearchingImage (X,Y,Width,Height:Integer);
var
  i,j:Integer;
  xmax,ymax:Integer;
begin
  xmax:=Width-1;
  ymax:=Height-1;
  Y:=Y-SearchingImage.Height;
  try
    for j:=0 to ymax do
    begin
      for i:=0 to xmax do
      begin
        SearchingImage.Image[j,i]:=SmallIntBuff[Y+j,X+i];
      end;
    end;
    SearchingImage.Width:=Width;
    SearchingImage.Height:=Height;
  except
  end;
end;

function CalculateBrightnessError (X,Y:Integer):Integer;
var
  i,j:Integer;
  xmax,ymax:Integer;
  res:SmallInt;
begin
  result:=0;
  xmax:=SearchingImage.Width-1;
  ymax:=SearchingImage.Height-1;
  try
    for j:=0 to ymax do
    begin
      for i:=0 to xmax do
      begin
        res:=SearchingImage.Image[j,i]-SmallIntBuff[Y+j,X+i];
        if (res<0) then 
          Dec(result,res) 
        else
          Inc(result,res);
      end;
    end;
  except
    result:=100000;
  end;
end;

function FindMinimumBrightnessErrorAtAll(var X,Y:Integer):Integer; // return minimum error
var
  i,j:Integer;
  xmax,ymax:Integer;
  CurrError:Integer;
begin
  result:=1000000;
  X:=algFrameWidth div 2;
  Y:=algFrameHeight div 2;
  xmax:=algFrameWidth-SearchingImage.Width-1;
  ymax:=algFrameHeight-SearchingImage.Height-1;
  for j:=0 to ymax do
  begin
    for i:=0 to xmax do
    begin
      CurrError:=CalculateBrightnessError(i,j);
      if (result>CurrError) then 
      begin
        result:=CurrError;
        X:=i; 
        Y:=j;
      end;
    end;
  end;
  Y:=Y+SearchingImage.Height;
end;

function FindMinimumBrightnessErrorAtXYR(X1,Y1,R:Integer;var X,Y:Integer):Integer; // return minimum error
var
  i,j:Integer;
  xmin,ymin:Integer;
  xmax,ymax:Integer;
  CurrError:Integer;
begin
  result:=1000000;
  Y1:=Y1-SearchingImage.Height+1;
  xmin:=Max(0,X1-R);
  ymin:=Max(0,Y1-R);
  xmax:=Min(algFrameWidth-SearchingImage.Width-1,X1+R);
  ymax:=Min(algFrameHeight-SearchingImage.Height-1,Y1+R);
  X:=algFrameWidth div 2;
  Y:=algFrameHeight div 2;
  for j:=ymin to ymax do
  begin
    for i:=xmin to xmax do
    begin
      CurrError:=CalculateBrightnessError(i,j);
      if (result>CurrError) then 
      begin
        result:=CurrError;
        X:=i; 
        Y:=j;
      end;
    end;
  end;
  Y:=Y+SearchingImage.Height-1;
end;

procedure MinThreshold (Threshold:SmallInt);
  procedure MinThresholdLine (var SmallIntArr:TLineSmallIntArray;Threshold:SmallInt);
  var
    i:Integer;
  begin
    for i:=0 to LastItemInLine do if (abs(SmallIntArr[i])<Threshold) then SmallIntArr[i]:=0;
  end;
begin
  MinThresholdLine (pLineSmallIntArray(@SmallIntBuff)^,Threshold);
end;

procedure ShowNormDelta (ZeroLevel:SmallInt);
  procedure ShowNormDeltaLine (var SmallIntArr:TLineSmallIntArray;ZeroLevel:SmallInt);
  var
    i:Integer;
    Value:SmallInt;
  begin
    for i:=0 to LastItemInLine do
    begin
      Value:=SmallIntArr[i]+ZeroLevel;
      if (Value<0) then SmallIntArr[i]:=0
      else if (Value>255) then SmallIntArr[i]:=255
      else SmallIntArr[i]:=Value;      
    end;
  end;
begin
  ShowNormDeltaLine(pLineSmallIntArray(@SmallIntBuff)^,ZeroLevel);
end;

procedure FindMinMax(var Minimum,Maximum:SmallInt);
  procedure FindMinMaxLine(const SmallIntArr:TLineSmallIntArray;var Minimum,Maximum:SmallInt);
  var
    i:Integer;
    MaxV,MinV,Value:SmallInt;
  begin
    MinV:=SmallIntArr[0];
    MaxV:=Minimum;
    for i:=1 to LastItemInLine do
    begin
      Value:=SmallIntArr[i];
      if (MaxV<Value) then MaxV:=Value
      else if (MinV>Value) then MinV:=Value;
    end;
    Minimum:=MinV;
    Maximum:=MaxV;
  end;
begin
  FindMinMaxLine(pLineSmallIntArray(@SmallIntBuff)^,Minimum,Maximum);
end;

procedure Differences01;
var
  y,x:SmallInt;
  tmpV:SmallInt;
begin
  TmpBuff:=SmallIntBuff;
  for x:=0 to LastCol do SmallIntBuff[0,x]:=0;
  for x:=0 to LastCol do SmallIntBuff[LastRow,x]:=0;
  for y:=0 to LastRow do SmallIntBuff[y,0]:=0;
  for y:=0 to LastRow do SmallIntBuff[y,LastCol]:=0;
  for y:=1 to LastRow-1 do
    for x:=1 to LastCol-1 do
    begin
      tmpV:=TmpBuff[y,x];
      SmallIntBuff[y,x]:=
        abs(tmpV-TmpBuff[y-1,x-1])+abs(tmpV-TmpBuff[y-1,x])+abs(tmpV-TmpBuff[y-1,x+1])+
        abs(tmpV-TmpBuff[y  ,x-1])+                         abs(tmpV-TmpBuff[y,x+1])+
        abs(tmpV-TmpBuff[y+1,x-1])+abs(tmpV-TmpBuff[y+1,x])+abs(tmpV-TmpBuff[y+1,x+1]);
    end;
end;

procedure Differences02;
var
  y,x:SmallInt;
  tmpV:SmallInt;
begin
  TmpBuff:=SmallIntBuff;
  for x:=0 to LastCol do SmallIntBuff[0,x]:=0;
  for x:=0 to LastCol do SmallIntBuff[LastRow,x]:=0;
  for y:=0 to LastRow do SmallIntBuff[y,0]:=0;
  for y:=0 to LastRow do SmallIntBuff[y,LastCol]:=0;
  for y:=1 to LastRow-1 do
    for x:=1 to LastCol-1 do
    begin
      tmpV:=TmpBuff[y,x];
      SmallIntBuff[y,x]:=tmpV*8-
        (TmpBuff[y-1,x-1]+TmpBuff[y-1,x]+TmpBuff[y-1,x+1]+
         TmpBuff[y  ,x-1]+TmpBuff[y,x+1]+
         TmpBuff[y+1,x-1]+TmpBuff[y+1,x]+TmpBuff[y+1,x+1]);
    end;
end;

procedure LoadRGBArray(const Data:p352x288RGBArray);
begin
  move(Data^,RGBBuff,SizeOf(RGBBuff));
end;

procedure StoreRGBArray(const Data:p352x288RGBArray);
begin
  move(RGBBuff,Data^,SizeOf(RGBBuff));
end;

procedure RGB2SmallIntArrayConvert; // ,   RGBBuff  
  procedure RGB2SmallIntLineArrayConvert (const RGBArr:TLineRGBArray;var SmallIntArr:TLineSmallIntArray);
  var
    i:Integer;
  begin
    for i:=0 to LastItemInLine do
    with RGBArr[i] do
      SmallIntArr[i]:=(g+b);
  end;
begin
  RGB2SmallIntLineArrayConvert(pLineRGBArray(@RGBBuff)^,pLineSmallIntArray(@SmallIntBuff)^);
end;

procedure SmallInt2RGBArrayConvert; // ,   SmallIntBuff  
  procedure SmallInt2RGBLineArrayConvert (const SmallIntArr:TLineSmallIntArray;var RGBArr:TLineRGBArray);
  var
    i:Integer;
    clr:SmallInt;
  begin
    for i:=0 to LastItemInLine do
    with RGBArr[i] do
    begin
      clr:=SmallIntArr[i] shr 1;
      r:=clr;
      g:=clr;
      b:=clr;
    end;
  end;
//var
  //i:Integer;
begin
  //   ,      ""
  //for i:=0 to algFrameWidth-1 do SmallIntBuff[10,i]:=255;
  SmallInt2RGBLineArrayConvert(pLineSmallIntArray(@SmallIntBuff)^,pLineRGBArray(@RGBBuff)^);
end;

initialization
  IsProcessing:=false;
finalization
end.
