unit ProcessingThread;

interface
uses Classes,DIB,FrameProcessing;
type
TOnFrameProcessedEvent = procedure (Sender: TObject; Bitmap:TDIB;const X1,Y1,X2,Y2:Integer) of object;
TFrameProcessing = class (TThread)
protected
  procedure Execute; override;
protected
  X1,Y1,X2,Y2:Integer;  // rectangle coordinates
  procedure DoEvent;
public
  Bitmap:TDIB;
  OnFrameProcessed:TOnFrameProcessedEvent;
  constructor Create(X1,Y1,X2,Y2:Integer);
  procedure ProcessImage(pBuffer: Pointer);
  destructor Destroy;override;
end;

implementation

{ TFrameProcessing }

constructor TFrameProcessing.Create;
begin
  inherited Create(True);
  Self.X1:=X1;
  Self.Y1:=Y1;
  Self.X2:=X2;
  Self.Y2:=Y2;
  Bitmap:=TDIB.Create;
  Bitmap.SetSize(algFrameWidth,algFrameHeight,24);
  FreeOnTerminate:=true;
end;

destructor TFrameProcessing.Destroy;
begin
  Bitmap.Free;
  Bitmap:=nil;
  inherited Destroy;
end;

procedure TFrameProcessing.DoEvent;
begin
  try OnFrameProcessed(Self,Bitmap,X1,Y1,X2,Y2) except end;
end;

procedure TFrameProcessing.Execute;
var
  NewX,NewY:Integer;
  DeltaX,DeltaY:Integer;
begin
  while not Terminated do
  begin
    if not IsProcessing then
    begin
      IsProcessing:=true;
      if Assigned(OnFrameProcessed) then
      begin

        // Init buffer
        if not Terminated then RGB2SmallIntArrayConvert;
        //        
        //FindMinimumBrightnessErrorAtAll(NewX,NewY);
        if not Terminated then FindMinimumBrightnessErrorAtXYR(X1,(algFrameHeight-Y1),10,NewX,NewY);
        DeltaX:=NewX-X1;
        DeltaY:=(algFrameHeight-Y1)-NewY-2;
        X1:=X1+DeltaX;
        Y1:=Y1+DeltaY;
        X2:=X2+DeltaX;
        Y2:=Y2+DeltaY;
        //    
        if not Terminated then FillSearchingImage(X1,(algFrameHeight-1)-Y1,X2-X1+1,Y2-Y1+1);
        if not Terminated then SmallInt2RGBArrayConvert;
        if not Terminated then StoreRGBArray(Bitmap.PBitsReadOnly);
      
        // return image to application
        if not Terminated then Synchronize(DoEvent);
      end;
      IsProcessing:=false;
    end;
    Suspend;
  end;
end;

procedure TFrameProcessing.ProcessImage(pBuffer: Pointer);
begin
  if IsProcessing then exit;
  // Init buffer
  LoadRGBArray(pBuffer);
  Resume;
end;

end.
