{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: main.PAS, released on 12 August 2004.

The Initial Developer of the Original Code is Alexey Nikitayev [nikitayev@rsu.ru]
All Rights Reserved.

Contributor(s):

Last Modified: 12 August 2004

You may retrieve the latest version of this file at the Project Opencomputer's home page,
located at http://opencomputer.narod.ru/

Known Issues:

Required: DelphiX, DSPack. http://opencomputer.narod.ru/programming.htm
-----------------------------------------------------------------------------}
unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DSUtil, StdCtrls, DSPack, DirectShow9, Menus, ExtCtrls, DXDraws,
  DIB, FrameProcessing, ProcessingThread;

type
  TVideoForm = class(TForm)
    FilterGraph: TFilterGraph;
    Filter: TFilter;
    SampleGrabber: TSampleGrabber;
    DXDraw: TDXDraw;
    FilterEmpty: TFilter;
    VisualizeTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure DXDrawInitializeSurface(Sender: TObject);
    procedure DXDrawMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DXDrawMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DXDrawMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure SampleGrabberBuffer(sender: TObject; SampleTime: Double;
      pBuffer: Pointer; BufferLen: Integer);
  private
    { Dclarations prives }
    Bitmap:TDIB;
    MousePressed:boolean;
    FindingFace:boolean; // /   
    X1,Y1,X2,Y2:Integer;
  public
    { Dclarations publiques }
    FrameWidth,FrameHeight:Integer;
    ProcessingThread: TFrameProcessing;
    function SelectRGB24Mode (Camera:TEnumMediaType;Width,Height:Integer):Integer;
    procedure GetDimensions (var Width,Height:Integer);
    procedure OnFrameProcessedEvent (Sender: TObject; Bitmap:TDIB;const X1,Y1,X2,Y2:Integer);
  end;

var
  VideoForm: TVideoForm;
  SysDev: TSysDevEnum;
implementation

{$R *.dfm}

procedure TVideoForm.FormCreate(Sender: TObject);
begin
  //DoubleBuffered:=false;
  DoubleBuffered:=true;
  Bitmap:=TDIB.Create;
  Bitmap.SetSize(algFrameWidth,algFrameHeight,24);
end;

procedure TVideoForm.FormShow(Sender: TObject);
var
  PinList: TPinList;
  VideoMediaTypes: TEnumMediaType;
  VideoFormatIndex: Integer;
begin
  SysDev:= TSysDevEnum.Create(CLSID_VideoInputDeviceCategory);
  FilterGraph.ClearGraph;
  FilterGraph.Active := false;
  Filter.BaseFilter.Moniker := SysDev.GetMoniker(0);
  FilterGraph.Active := true;
  PinList := TPinList.Create(Filter as IBaseFilter);
  VideoMediaTypes := TEnumMediaType.Create;
  VideoMediaTypes.Assign(PinList.First);
  VideoFormatIndex:=SelectRGB24Mode(VideoMediaTypes,algFrameWidth,algFrameHeight);

  // configure output Video media type
  if VideoFormatIndex <> -1 then
    with (PinList.First as IAMStreamConfig) do
      SetFormat(VideoMediaTypes.Items[VideoFormatIndex].AMMediaType^);
  FreeAndNil(PinList);

  if (VideoFormatIndex = -1) then
  begin
    Caption:='PlayCap: 320x240x24 Does not supported';
    exit;
  end;
  
  with FilterGraph as ICaptureGraphBuilder2 do
    RenderStream(nil, nil, Filter as IBaseFilter, SampleGrabber as IBaseFilter, FilterEmpty as IbaseFilter);
  FilterGraph.Play;
  GetDimensions(FrameWidth,FrameHeight);
  Caption:=Format('PlayCap [%dx%d]',[FrameWidth,FrameHeight]);
  if (FrameWidth<>algFrameWidth) or (FrameHeight<>algFrameHeight) then
  begin
    Caption:=Caption+' You must set these values to constants'; 
    SampleGrabber.OnBuffer:=nil;
    exit;
  end;
  MousePressed:=false;
  X1:=0;
  Y1:=0;
  X2:=0;
  Y2:=0;
end;

procedure TVideoForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if Assigned(ProcessingThread) then ProcessingThread.Terminate;
  SysDev.Free;
  FilterGraph.ClearGraph;
  FilterGraph.Active := false;
  FreeAndNil(Bitmap);
end;

procedure TVideoForm.GetDimensions(var Width, Height: Integer);
var
  hr         : HRESULT;
  BMIHeader  : TBitmapInfoHeader;
  AMediaType : TAMMediaType;
begin
  Width:=0;
  Height:=0;
  if not assigned(SampleGrabber) then exit;
  if not assigned(SampleGrabber.SampleGrabber) then exit;
  hr := SampleGrabber.SampleGrabber.GetConnectedMediaType(AMediaType);
  if (hr <> S_OK) then exit;
  if IsEqualGUID(AMediaType.majortype, MEDIATYPE_Video) then
  begin
    case Amediatype.formattype.D1 of
      $05589F80: BMIHeader := PVideoInfoHeader(Amediatype.pbFormat)^.bmiHeader;
      $F72A76A0: BMIHeader := PVideoInfoHeader2(Amediatype.pbFormat)^.bmiHeader;
    end;
    Width:=BMIHeader.biWidth;
    Height:=BMIHeader.biHeight;
  end;
end;

procedure TVideoForm.DXDrawInitializeSurface(Sender: TObject);
begin
  DXDraw.Surface.SetSize(algFrameWidth,algFrameHeight);
  DXDraw.Surface.Canvas.Pen.Color:=clWhite;
  DXDraw.Surface.Canvas.Pen.Mode:=pmCopy;
  DXDraw.Surface.Canvas.Pen.Style:=psSolid;
  DXDraw.Surface.Canvas.Brush.Style:=bsClear;
  DXDraw.Surface.Canvas.Release;

  VisualizeTimer.Enabled:=true;
end;

procedure TVideoForm.DXDrawMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FindingFace:=false;
  if (ssLeft in Shift) then
  begin
    MousePressed:=true;
    TranslateXY(X,Y,DXDraw.Width,DXDraw.Height);
    X1:=X;
    Y1:=Y;
    X2:=X;
    Y2:=Y;
  end
  else
  if (ssRight in Shift) then
  begin
    MousePressed:=false;
    X1:=0;
    Y1:=0;
    X2:=0;
    Y2:=0;
    ProcessingThread.Terminate;
    ProcessingThread:=nil;
  end;
end;

procedure TVideoForm.DXDrawMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if MousePressed then
  begin
    TranslateXY(X,Y,DXDraw.Width,DXDraw.Height);
    X2:=X;
    Y2:=Y;
  end;
end;

procedure TVideoForm.DXDrawMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if MousePressed then
  begin
    MousePressed:=false;
    TranslateXY(X,Y,DXDraw.Width,DXDraw.Height);
    X2:=X;
    Y2:=Y;
    if (X2<X1) then SwapAB(X1,X2);
    if (Y2<Y1) then SwapAB(Y1,Y2);
    FindingFace:=true;
    //   
    FillSearchingImage(X1,(FrameHeight-1)-Y1,X2-X1+1,Y2-Y1+1);
    // create processing thread
    ProcessingThread:= TFrameProcessing.Create(X1,Y1,X2,Y2);
    ProcessingThread.OnFrameProcessed:=OnFrameProcessedEvent;
  end;
end;

procedure TVideoForm.SampleGrabberBuffer(sender: TObject;
  SampleTime: Double; pBuffer: Pointer; BufferLen: Integer);
begin
  if (DXDraw.Surface.IDDSurface=nil) then exit;
  if (not Assigned(Bitmap)) or (not DXDraw.CanDraw) then exit;
  try
    if FindingFace then
    begin
      if Assigned(ProcessingThread) then ProcessingThread.ProcessImage(pBuffer);
    end
    else
    begin
      // Init buffer
      LoadRGBArray(pBuffer);
      //RGB2SmallIntArrayConvert;
      //SmallInt2RGBArrayConvert;
      StoreRGBArray(Bitmap.PBitsReadOnly);

      if (DXDraw.Surface.IDDSurface<>nil) then DXDraw.Surface.Canvas.Draw(0,0,Bitmap);
      if (DXDraw.Surface.IDDSurface<>nil) then DXDraw.Surface.Canvas.Rectangle(X1,Y1,X2,Y2);
      if (DXDraw.Surface.IDDSurface<>nil) then DXDraw.Surface.Canvas.Release;
      if (DXDraw.Surface.IDDSurface<>nil) then DXDraw.Flip;
      Application.ProcessMessages;
    end;
  finally
  end;
end;

procedure TVideoForm.OnFrameProcessedEvent(Sender: TObject; Bitmap: TDIB;
  const X1, Y1, X2, Y2: Integer);
begin
  if (DXDraw.Surface.IDDSurface<>nil) then DXDraw.Surface.Canvas.Draw(0,0,Bitmap);
  if (DXDraw.Surface.IDDSurface<>nil) then DXDraw.Surface.Canvas.Rectangle(X1,Y1,X2,Y2);
  if (DXDraw.Surface.IDDSurface<>nil) then DXDraw.Surface.Canvas.Release;
  if (DXDraw.Surface.IDDSurface<>nil) then DXDraw.Flip;
end;

function TVideoForm.SelectRGB24Mode(Camera:TEnumMediaType;Width, Height: Integer):Integer;
var
  i:Integer;
begin
  result:=-1;
  for i:=0 to Camera.Count-1 do
  with Camera.Items[i] do
  begin
    if IsEqualGUID(AMMediaType.majortype,MEDIATYPE_Video) then
      if IsEqualGUID(AMMediaType.subtype,MEDIASUBTYPE_RGB24) then
        if IsEqualGUID(AMMediaType.formattype,FORMAT_VideoInfo) then
        begin
          if ((AMMediaType.cbFormat > 0) and assigned(AMMediaType.pbFormat)) then
          with PVideoInfoHeader(AMMediaType.pbFormat)^.bmiHeader do
          if ((biWidth=Width) and (Height=biHeight) and (biBitCount=24)) then
          begin
            result:=i;
            break;
          end;
        end
  end;
end;

end.
